@@ -1,23 +1,22 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the zim.formats module.'''
 
-from __future__ import with_statement
+
 
 
 import tests
 
 from zim.formats import *
 from zim.fs import File
-from zim.notebook import Path, Link
+from zim.notebook import Path
 from zim.parsing import link_type
 from zim.templates import Template
 
 
 if not ElementTreeModule.__name__.endswith('cElementTree'):
-	print 'WARNING: using ElementTree instead of cElementTree'
+	print('WARNING: using ElementTree instead of cElementTree')
 
 
 class TestFormatMixin(object):
@@ -36,7 +35,7 @@ class TestFormatMixin(object):
 
 	def testFormatInfo(self):
 		for key in ('name', 'desc', 'mimetype', 'extension'):
-			self.assertIsInstance(self.format.info[key], basestring,
+			self.assertIsInstance(self.format.info[key], str,
 				msg='Invalid key "%s" in format info' % key)
 
 		for key in ('native', 'import', 'export'):
@@ -61,7 +60,8 @@ class TestFormatMixin(object):
 		# No absolute paths ended up in reference
 		pwd = Dir('.')
 		self.assertFalse(pwd.path in text, 'Absolute path ended up in reference')
-		self.assertFalse(pwd.user_path in text, 'Absolute path ended up in reference')
+		if pwd.user_path is not None:
+			self.assertFalse(pwd.user_path in text, 'Absolute path ended up in reference')
 
 		return text
 
@@ -75,7 +75,7 @@ class TestFormatMixin(object):
 		linker = StubLinker(Dir('tests/data/formats'))
 		dumper = self.format.Dumper(linker=linker)
 		result = ''.join(dumper.dump(reftree))
-		#~ print '\n' + '>'*80 + '\n' + result + '\n' + '<'*80 + '\n'
+		#~ print('\n' + '>'*80 + '\n' + result + '\n' + '<'*80 + '\n')
 		self.assertMultiLineEqual(result, wanted)
 		self.assertNoTextMissing(result, reftree)
 
@@ -85,7 +85,7 @@ class TestFormatMixin(object):
 		# partial dumper
 		parttree = tests.new_parsetree_from_xml("<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial=\"True\">try these <strong>bold</strong>, <emphasis>italic</emphasis></zim-tree>")
 		result = ''.join(dumper.dump(parttree))
-		#~ print ">>>%s<<<" % result
+		#~ print(">>>%s<<<" % result)
 		self.assertFalse(result.endswith('\n')) # partial should not end with "\n"
 
 		# Parser
@@ -101,7 +101,7 @@ class TestFormatMixin(object):
 				# Quick check that we got back *something*
 			string = ''.join(dumper.dump(result))
 				# now we may have loss of formatting, but text should all be there
-				#~ print '\n' + '>'*80 + '\n' + string + '\n' + '<'*80 + '\n'
+				#~ print('\n' + '>'*80 + '\n' + string + '\n' + '<'*80 + '\n')
 			self.assertNoTextMissing(string, reftree)
 
 	_nonalpha_re = re.compile('\W')
@@ -112,7 +112,7 @@ class TestFormatMixin(object):
 		is preserved.
 		'''
 		# TODO how to handle objects ??
-		assert isinstance(text, basestring)
+		assert isinstance(text, str)
 		offset = 0
 		for elt in tree._etree.iter():
 			if elt.tag == 'img':
@@ -132,11 +132,11 @@ class TestFormatMixin(object):
 					continue
 
 				for piece in wanted.strip().split():
-					#~ print "| >>%s<< @ offset %i" % (piece, offset)
+					#~ print("| >>%s<< @ offset %i" % (piece, offset))
 					try:
 						start = text.index(piece, offset)
 					except ValueError:
-						self.fail('Could not find text piece "%s" in text after offset %i\n>>>%s<<<' % (piece, offset, text[offset:offset+100]))
+						self.fail('Could not find text piece "%s" in text after offset %i\n>>>%s<<<' % (piece, offset, text[offset:offset + 100]))
 					else:
 						offset = start + len(piece)
 
@@ -267,12 +267,6 @@ class TestParseTree(tests.TestCase):
 			tree = ParseTree().fromstring(xml)
 			self.assertEqual(tree.get_ends_with_newline(), newline)
 
-	def testGetObjects(self):
-		xml = File('tests/data/formats/parsetree.xml').read().rstrip('\n')
-		tree = tests.new_parsetree_from_xml(xml)
-		objects = list(tree.get_objects())
-		self.assertTrue(len(objects) >= 2)
-
 	def testFindall(self):
 		tree = ParseTree().fromstring(self.xml)
 		wanted = [
@@ -337,33 +331,18 @@ class TestWikiFormat(TestTextFormat):
 
 	def setUp(self):
 		self.format = get_format('wiki')
-		notebook = tests.new_notebook()
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 		self.page = notebook.get_page(Path('Foo'))
 
-	#~ def testHeaders(self):
-		#~ text = '''\
-		#~ Content-Type: text/x-zim-wiki
-		#~ Wiki-Format: zim 0.26
-		#~ Creation-Date: Unkown
-		#~ Modification-Date: Wed, 06 Aug 2008 22:17:29 +0200
-		#~
-		#~ foo bar
-		#~ '''
-		#~ tree = self.format.Parser().parse(text)
-		#~ print '>>>\n'+tostring(tree)+'\n<<<\n'
-		#~ self.assertEquals(tree.getroot().attrib['Content-Type'], 'text/x-zim-wiki')
-		#~ output = self.format.Dumper().dump(tree)
-		#~ self.assertEqual(output, text.splitlines(True))
-
 	def testUnicodeBullet(self):
 		'''Test support for unicode bullets in source'''
-		input = u'''\
+		input = '''\
 A list
 • foo
 	• bar
 	• baz
 '''
-		text = u'''\
+		text = '''\
 A list
 * foo
 	* bar
@@ -384,14 +363,12 @@ A list
 		for elt in tree.findall(LINK):
 			self.assertTrue(elt.gettext())
 			self.assertTrue(elt.get('href'))
-			link = Link(self.page, **elt.attrib)
-			self.assertEqual(elt.attrib['href'], link.href)
 			found += 1
 		self.assertEqual(found, 3)
 
 	def testBackward(self):
 		'''Test backward compatibility for wiki format'''
-		input = u'''\
+		input = '''\
 test 1 2 3
 
 	Some Verbatim block
@@ -399,7 +376,7 @@ test 1 2 3
 
 test 4 5 6
 '''
-		wanted = u'''\
+		wanted = '''\
 test 1 2 3
 
 \'''
@@ -429,12 +406,12 @@ test 4 5 6
 				wanted = text
 
 			tree = self.format.Parser().parse(text)
-			#~ print '>>>\n' + tree.tostring() + '\n<<<'
+			#~ print('>>>\n' + tree.tostring() + '\n<<<')
 			self.assertEqual(tree.tostring(), xml)
 
 			lines = self.format.Dumper().dump(tree)
 			result = ''.join(lines)
-			#~ print '>>>\n' + result + '<<<'
+			#~ print('>>>\n' + result + '<<<')
 			self.assertEqual(result, wanted)
 
 
@@ -485,9 +462,23 @@ C. hmmm
 		check(text, xml)
 
 
-		# Inconsistent lists
-		# ( If first item is number, make all items numbered in sequence
-		#   Otherwise numers will be turned into bullets )
+		# Incosistent list with different checkbox types
+		text = '''\
+* foo
+[ ] bar
+* dus
+'''
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree><p><ul><li bullet="*">foo</li><li bullet="unchecked-box">bar</li><li bullet="*">dus</li></ul></p></zim-tree>'''
+		wanted = '''\
+* foo
+[ ] bar
+* dus
+'''
+		check(text, xml, wanted)
+
+		# Inconsistent lists get broken in multiple lists
 		text = '''\
 1. foo
 4. bar
@@ -496,12 +487,12 @@ a. dus
 '''
 		xml = '''\
 <?xml version='1.0' encoding='utf-8'?>
-<zim-tree><p><ol start="1"><li>foo</li><li>bar</li><li>hmmm</li><li>dus</li></ol></p></zim-tree>'''
+<zim-tree><p><ol start="1"><li>foo</li><li>bar</li></ol><ul><li bullet="*">hmmm</li></ul><ol start="a"><li>dus</li></ol></p></zim-tree>'''
 		wanted = '''\
 1. foo
 2. bar
-3. hmmm
-4. dus
+* hmmm
+a. dus
 '''
 		check(text, xml, wanted)
 
@@ -513,15 +504,16 @@ a. hmmm
 '''
 		xml = '''\
 <?xml version='1.0' encoding='utf-8'?>
-<zim-tree><p><ul><li bullet="*">foo</li><li bullet="*">bar</li><li bullet="*">hmmm</li><li bullet="*">dus</li></ul></p></zim-tree>'''
+<zim-tree><p><ul><li bullet="*">foo</li></ul><ol start="4"><li>bar</li><li>hmmm</li></ol><ul><li bullet="*">dus</li></ul></p></zim-tree>'''
 		wanted = '''\
 * foo
-* bar
-* hmmm
+4. bar
+5. hmmm
 * dus
 '''
 		check(text, xml, wanted)
 
+
 		# Mixed sub-list
 		text = '''\
 * foo
@@ -595,15 +587,22 @@ hmmm
 '''
 		tree = ParseTree()
 		tree.fromstring(xml)
-		text = ''.join( self.format.Dumper().dump(tree) )
+		text = ''.join(self.format.Dumper().dump(tree))
 		self.assertEqual(text, wanted)
 
+	def testStringEscapeDoesNotGetEvaluated(self):
+		text = "this is not a newline: \\name\n This is not a tab: \\tab \n"
+		tree = self.format.Parser().parse(text)
+		#~ print tree.tostring()
+		output = self.format.Dumper().dump(tree)
+		self.assertEqual(''.join(output), text)
+
 
 class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 
 	def setUp(self):
 		self.format = get_format('html')
-		notebook = tests.new_notebook()
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 		self.page = notebook.get_page(Path('Foo'))
 
 	def testEncoding(self):
@@ -622,9 +621,9 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 	def testEmptyLines(self):
 		builder = ParseTreeBuilder()
 		builder.start(FORMATTEDTEXT)
-		builder.append(HEADING, {'level':1}, 'head1')
+		builder.append(HEADING, {'level': 1}, 'head1')
 		builder.text('\n')
-		builder.append(HEADING, {'level':2}, 'head2')
+		builder.append(HEADING, {'level': 2}, 'head2')
 		builder.end(FORMATTEDTEXT)
 		tree = builder.get_parsetree()
 
@@ -703,30 +702,11 @@ class TestRstFormat(tests.TestCase, TestFormatMixin):
 		self.format = get_format('rst')
 
 
-class LatexLoggingFilter(tests.LoggingFilter):
-
-	logger = 'zim.formats.latex'
-	message = ('No document type set in template', 'Could not find latex equation')
-
-
 class TestLatexFormat(tests.TestCase, TestFormatMixin):
 
 	def setUp(self):
 		self.format = get_format('latex')
 
-	def testFormat(self):
-		with LatexLoggingFilter():
-			TestFormatMixin.testFormat(self)
-
-	def testFormatReference(self):
-		# Double check reference did not get broken in updating
-		text = self.getReferenceData()
-
-		# Inlined equation is there
-		self.assertFalse('equation001.png' in text, 'This equation should be inlined')
-		self.assertTrue(r'\begin{math}' in text)
-		self.assertTrue(r'\end{math}' in text)
-
 	def testEncode(self):
 		'''test the escaping of certain characters'''
 		format = get_format('latex')
@@ -738,9 +718,9 @@ class TestLatexFormat(tests.TestCase, TestFormatMixin):
 	def testDocumentType(self):
 		builder = ParseTreeBuilder()
 		builder.start(FORMATTEDTEXT)
-		builder.append(HEADING, {'level':1}, 'head1')
+		builder.append(HEADING, {'level': 1}, 'head1')
 		builder.text('\n')
-		builder.append(HEADING, {'level':2}, 'head2')
+		builder.append(HEADING, {'level': 2}, 'head2')
 		builder.end(FORMATTEDTEXT)
 		tree = builder.get_parsetree()
 
@@ -819,10 +799,34 @@ grrr
    .
 </zim-tree>'''
 
-		# For some reason this does not work with cElementTree.XMLBuilder ...
-		from xml.etree.ElementTree import XMLTreeBuilder
-		builder = XMLTreeBuilder(target=OldParseTreeBuilder())
+		from xml.etree.ElementTree import XMLParser
+		builder = XMLParser(target=OldParseTreeBuilder())
 		builder.feed(input)
 		root = builder.close()
 		tree = ParseTree(root)
 		self.assertEqual(tree.tostring(), wanted)
+
+
+class TestParseHeaderLines(tests.TestCase):
+
+	def runTest(self):
+		text = '''\
+Content-Type: text/x-zim-wiki
+Wiki-Format: zim 0.4
+X-Foo: Some text
+	here
+Creation-Date: 2010-12-14T14:15:09.134955
+
+Blaat
+'''
+		body, meta = parse_header_lines(text)
+		self.assertEqual(dict(meta), {
+			'Content-Type': 'text/x-zim-wiki',
+			'Wiki-Format': 'zim 0.4',
+			'Creation-Date': '2010-12-14T14:15:09.134955',
+			'X-Foo': 'Some text\nhere'
+		})
+		self.assertEqual(body, 'Blaat\n')
+
+		out = dump_header_lines(meta)
+		self.assertEqual(out + '\nBlaat\n', text)
